//
// FlagList.cpp
// asommers 10-29-96
// asommers 8-5-98
//
// copyright 1998, origin systems
//

#include "First.h"
#include "FlagList.h"

#include "Iff.h"
//#include "Mcp.h"

//
// Flag Implementation
//

FlagList::Flag::Flag (void)
{
	flagCrc = 0;
	typeCrc = 0;
	value   = 0;
}

FlagList::Flag::~Flag (void)
{
}

//
// FlagList Implementation
//

FlagList::FlagList (int list_size)
{
	flags = NEW Flag [ list_size ];
	size = list_size;

	numFlags = 0;
}

FlagList::FlagList (Iff& iff, int num_flags)
{
	char* block = iff.read_char (sizeof (Flag) * num_flags);
	flags = NEW Flag [num_flags];

	memcpy (flags, block, sizeof (Flag) * num_flags);
	delete [] block;

	size     = num_flags;
	numFlags = num_flags;
}

FlagList::~FlagList (void)
{
	delete [] flags;
	flags = NULL;
	
	size = 0;
	numFlags = 0;
}

FlagList::FlagList (const FlagList& rhs)
{
	size     = rhs.size;
	numFlags = rhs.numFlags;

	flags    = NEW Flag [numFlags];
	memcpy (flags, rhs.flags, sizeof (Flag) * numFlags);
}

void FlagList::resizeUp (void)
{
	if (!size)
		size = FLAGLIST_DEFAULT_SIZE;
	else
		size *= 2;

	Flag* newFlags = NEW Flag [size];

	if (numFlags > 0)
		memcpy (newFlags, flags, sizeof (Flag) * numFlags);

	delete [] flags;
	flags = newFlags;
}

void FlagList::addFlag ( u_int32 crc, u_int32 type, int32 value )
{
	if (numFlags >= size)
		resizeUp ();

	// Add the flag
	flags [ numFlags ].flagCrc = crc;
	flags [ numFlags ].typeCrc = type;
	flags [ numFlags ].value = value;

	numFlags++;
}

int32 FlagList::getFlagIndex (u_int32 crc, bool req) const
// Used during the fixup process, scans flags for the crc and
//   returns the index
{
	int i = 0;
	while (i < numFlags)
	{
		if (flags [i].flagCrc == crc)
			return i;

		i++;
	}

	// the flag wasn't found -- is it required?
	if (req)
		Fatal ("FlagList::getFlagIndex - index not found for crc %x\n", crc);

	return FLAGLIST_INDEX_INVALID;
}
//
//
//

int32 FlagList::getCrc ( int32 flag_num ) const
{
	FATAL ( ((flag_num < 0) || (flag_num >= numFlags)), ("FlagList::getCrc - %d is out of bounds\n", flag_num ));

	return flags [ flag_num ].flagCrc;
}

int32 FlagList::getValue ( int32 flag_num ) const
{
	FATAL ( ((flag_num < 0) || (flag_num >= numFlags)), ("FlagList::getValue - %d is out of bounds\n", flag_num ));

	return flags [ flag_num ].value;
}

void FlagList::setValue ( int32 flag_num, int32 value )
{
	FATAL ( ((flag_num < 0) || (flag_num >= numFlags)), ("FlagList::setFlag - %d is out of bounds\n", flag_num ));

	flags [ flag_num ].value = value;
}

void FlagList::addToFlag ( int32 flag_num, int32 value )
{
	FATAL ( ((flag_num < 0) || (flag_num >= numFlags)), ("FlagList::addToFlag - %d is out of bounds\n", flag_num ));

	flags [ flag_num ].value += value;
}

void FlagList::print (void)
{
	DPRINT (("\n"));

	for ( int i = 0; i < getNumFlags (); i++ )
	{
//		DPRINT (("%s (%x) : %d\n", CMcp::getNameFromCrc ( flags [ i ].flagCrc ), flags [ i ].flagCrc, flags [ i ].value ));
	}
}

int32 FlagList::getValueFromCrc (u_int32 crc) const
{
	return getValue (getFlagIndex (crc));
}

void FlagList::setValueFromCrc (u_int32 crc, int32 value)
{
	setValue (getFlagIndex (crc), value);
}

void FlagList::addValueFromCrc (u_int32 crc, int32 value)
{
	addToFlag (getFlagIndex (crc), value);
}

void FlagList::save (Iff& iff)
{
	iff.appendForm (TAG (F,L,A,G));

		iff.appendChunk (TAG (D,A,T,A));

			Iff__appendToChunk (iff, numFlags);
			iff.appendToChunk (flags, sizeof (Flag) * numFlags);

	iff.exitForm ();
}

void FlagList::load (Iff& iff)
{
	iff.seekForm (TAG (F,L,A,G));

		iff.seekChunk (TAG (D,A,T,A));

			numFlags = iff.read_int32 ();
			char* data = iff.read_char (sizeof (Flag) * numFlags);
			memcpy (flags, data, sizeof (Flag) * numFlags);
			delete [] data;

			size     = numFlags;
			numFlags = numFlags;

		iff.exitChunk ();

	iff.exitForm ();
}

// END
